@@ -1,27 +1,32 @@
 /*
-# MySQL Connector/Python - MySQL driver written in Python.
-# Copyright (c) 2014, 2016, Oracle and/or its affiliates. All rights reserved.
-
-# MySQL Connector/Python is licensed under the terms of the GPLv2
-# <http://www.gnu.org/licenses/old-licenses/gpl-2.0.html>, like most
-# MySQL Connectors. There are special exceptions to the terms and
-# conditions of the GPLv2 as it is applied to this software, see the
-# FOSS License Exception
-# <http://www.mysql.com/about/legal/licensing/foss-exception.html>.
-#
-# This program is free software; you can redistribute it and/or modify
-# it under the terms of the GNU General Public License as published by
-# the Free Software Foundation.
-#
-# This program is distributed in the hope that it will be useful,
-# but WITHOUT ANY WARRANTY; without even the implied warranty of
-# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-# GNU General Public License for more details.
-#
-# You should have received a copy of the GNU General Public License
-# along with this program; if not, write to the Free Software
-# Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA
-*/
+ * Copyright (c) 2014, 2017, Oracle and/or its affiliates. All rights reserved.
+ *
+ * This program is free software; you can redistribute it and/or modify
+ * it under the terms of the GNU General Public License, version 2.0, as
+ * published by the Free Software Foundation.
+ *
+ * This program is also distributed with certain software (including
+ * but not limited to OpenSSL) that is licensed under separate terms,
+ * as designated in a particular file or component or in included license
+ * documentation.  The authors of MySQL hereby grant you an
+ * additional permission to link the program and your derivative works
+ * with the separately licensed software that they have included with
+ * MySQL.
+ *
+ * Without limiting anything contained in the foregoing, this file,
+ * which is part of MySQL Connector/Python, is also subject to the
+ * Universal FOSS Exception, version 1.0, a copy of which can be found at
+ * http://oss.oracle.com/licenses/universal-foss-exception.
+ *
+ * This program is distributed in the hope that it will be useful, but
+ * WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
+ * See the GNU General Public License, version 2.0, for more details.
+ *
+ * You should have received a copy of the GNU General Public License
+ * along with this program; if not, write to the Free Software Foundation, Inc.,
+ * 51 Franklin St, Fifth Floor, Boston, MA 02110-1301  USA
+ */
 
 #include <stdio.h>
 #include <stdlib.h>
@@ -30,6 +35,9 @@
 #include <Python.h>
 #include <datetime.h>
 
+#ifdef MS_WINDOWS
+#include <windows.h>
+#endif
 #include <mysql.h>
 
 #include "catch23.h"
@@ -363,7 +371,7 @@ MySQL_init(MySQL *self, PyObject *args, PyObject *kwds)
 
     // Initialization expect -1 when parsing arguments failed
     if (!PyArg_ParseTupleAndKeywords(args, kwds,
-                                     "|O!O!O!O!O!O", kwlist,
+                                     "|O!O!O!O!O!O!", kwlist,
                                      &PyBool_Type, &self->buffered_at_connect,
                                      &PyBool_Type, &self->raw_at_connect,
                                      &PyStringType, &self->charset_name,
@@ -393,6 +401,10 @@ MySQL_init(MySQL *self, PyObject *args, PyObject *kwds)
 
     if (auth_plugin)
     {
+        if (strcmp(PyStringAsString(auth_plugin), "") == 0)
+        {
+            auth_plugin= Py_None;
+        }
         if (auth_plugin != Py_None)
         {
             tmp= self->auth_plugin;
@@ -1025,32 +1037,45 @@ MySQL_commit(MySQL *self)
 PyObject*
 MySQL_connect(MySQL *self, PyObject *args, PyObject *kwds)
 {
-	char *host= NULL, *user= NULL, *password= NULL, *database= NULL,
-	     *unix_socket= NULL;
+	char *host= NULL, *user= NULL, *database= NULL, *unix_socket= NULL;
 	char *ssl_ca= NULL, *ssl_cert= NULL, *ssl_key= NULL;
-	PyObject *charset_name, *compress, *ssl_verify_cert;
+	PyObject *charset_name, *compress, *ssl_verify_cert, *password, *ssl_disabled;
 	const char* auth_plugin;
 	unsigned long client_flags= 0;
 	unsigned int port= 3306, tmp_uint;
 	unsigned int protocol= 0;
-	my_bool abool;
+#if MYSQL_VERSION_ID >= 50711
+	unsigned int ssl_mode;
+#endif
+#if MYSQL_VERSION_ID >= 80001
+	bool abool;
+	bool ssl_enabled= 0;
+#else
+  my_bool abool;
+  my_bool ssl_enabled= 0;
+#endif
 	MYSQL *res;
 
 	static char *kwlist[]=
 	{
 	    "host", "user", "password", "database",
 		"port", "unix_socket", "client_flags",
-		"ssl_ca", "ssl_cert", "ssl_key", "ssl_verify_cert",
+		"ssl_ca", "ssl_cert", "ssl_key", "ssl_verify_cert", "ssl_disabled",
 		"compress",
 		NULL
     };
 
-    if (!PyArg_ParseTupleAndKeywords(args, kwds, "|zzzzkzkzzzO!O!", kwlist,
+#ifdef PY3
+    if (!PyArg_ParseTupleAndKeywords(args, kwds, "|zzzzkzkzzzO!O!O!", kwlist,
+#else
+    if (!PyArg_ParseTupleAndKeywords(args, kwds, "|zzOzkzkzzzO!O!O!", kwlist,
+#endif
                                      &host, &user, &password, &database,
                                      &port, &unix_socket,
                                      &client_flags,
                                      &ssl_ca, &ssl_cert, &ssl_key,
                                      &PyBool_Type, &ssl_verify_cert,
+                                     &PyBool_Type, &ssl_disabled,
                                      &PyBool_Type, &compress))
     {
         return NULL;
@@ -1105,33 +1130,24 @@ MySQL_connect(MySQL *self, PyObject *args, PyObject *kwds)
     mysql_options(&self->session, MYSQL_OPT_READ_TIMEOUT, (char*)&tmp_uint);
     mysql_options(&self->session, MYSQL_OPT_WRITE_TIMEOUT, (char*)&tmp_uint);
 
-    if (ssl_ca || ssl_cert || ssl_key) {
-#if MYSQL_VERSION_ID > 50703 && MYSQL_VERSION_ID < 50711
-        printf(">>>> %d\n", MYSQL_VERSION_ID);
-        {
-            abool= 1;
-            mysql_options(&self->session, MYSQL_OPT_SSL_ENFORCE, (char*)&abool);
-        }
-#endif
-#if MYSQL_VERSION_ID >= 50711
-        {
-            mysql_options(&self->session, MYSQL_OPT_SSL_MODE, SSL_MODE_REQUIRED);
-        }
-#endif
-
+    if (ssl_disabled && ssl_disabled == Py_False) {
+        ssl_enabled= 1;
+        client_flags |= CLIENT_SSL;
         if (ssl_verify_cert && ssl_verify_cert == Py_True)
         {
 #if MYSQL_VERSION_ID >= 50711
-            {
-                mysql_options(&self->session, MYSQL_OPT_SSL_MODE, SSL_MODE_VERIFY_IDENTITY);
-            }
+            ssl_mode= SSL_MODE_VERIFY_IDENTITY;
+            mysql_options(&self->session, MYSQL_OPT_SSL_MODE, &ssl_mode);
 #else
-            {
-                abool= 1;
-                mysql_options(&self->session,
-                              MYSQL_OPT_SSL_VERIFY_SERVER_CERT, (char*)&abool);
-            }
+            abool= 1;
+#if MYSQL_VERSION_ID > 50703
+            mysql_options(&self->session, MYSQL_OPT_SSL_ENFORCE, (char*)&abool);
 #endif
+            mysql_options(&self->session,
+                          MYSQL_OPT_SSL_VERIFY_SERVER_CERT, (char*)&abool);
+#endif
+        } else {
+          ssl_ca= NULL;
         }
         mysql_ssl_set(&self->session, ssl_key, ssl_cert, ssl_ca, NULL, NULL);
     } else {
@@ -1144,14 +1160,33 @@ MySQL_connect(MySQL *self, PyObject *args, PyObject *kwds)
 #endif
 #if MYSQL_VERSION_ID >= 50711
         {
-            mysql_options(&self->session, MYSQL_OPT_SSL_ENFORCE, SSL_MODE_DISABLED);
+            ssl_mode= SSL_MODE_DISABLED;
+            mysql_options(&self->session, MYSQL_OPT_SSL_MODE, &ssl_mode);
         }
 #endif
     }
 
+    Py_END_ALLOW_THREADS
     if (PyString_Check(self->auth_plugin)) {
         auth_plugin= PyStringAsString(self->auth_plugin);
         mysql_options(&self->session, MYSQL_DEFAULT_AUTH, auth_plugin);
+        if (strcmp(auth_plugin, "sha256_password") == 0 && !ssl_enabled)
+        {
+            PyObject *exc_type= MySQLInterfaceError;
+            PyObject *err_no= PyInt_FromLong(2002);
+            PyObject *err_msg= PyStringFromString("sha256_password requires SSL");
+            PyObject *err_obj= NULL;
+            err_obj= PyObject_CallFunctionObjArgs(exc_type, err_msg, NULL);
+            PyObject_SetAttr(err_obj, PyStringFromString("sqlstate"), Py_None);
+            PyObject_SetAttr(err_obj, PyStringFromString("errno"), err_no);
+            PyObject_SetAttr(err_obj, PyStringFromString("msg"), err_msg);
+            PyErr_SetObject(exc_type, err_obj);
+            Py_XDECREF(exc_type);
+            Py_XDECREF(err_no);
+            Py_XDECREF(err_msg);
+            return NULL;
+        }
+
         if (strcmp(auth_plugin, "mysql_clear_password") == 0)
         {
             abool= 1;
@@ -1159,6 +1194,7 @@ MySQL_connect(MySQL *self, PyObject *args, PyObject *kwds)
                           (char*)&abool);
         }
     }
+    Py_BEGIN_ALLOW_THREADS
 
     if (database && strlen(database) == 0)
     {
@@ -1170,9 +1206,31 @@ MySQL_connect(MySQL *self, PyObject *args, PyObject *kwds)
         client_flags= client_flags & ~CLIENT_CONNECT_WITH_DB;
     }
 
-    res= mysql_real_connect(&self->session,
-                            host, user, password, database,
-    						port, unix_socket, client_flags);
+    if (client_flags & CLIENT_LOCAL_FILES) {
+        abool= 1;
+        mysql_options(&self->session, MYSQL_OPT_LOCAL_INFILE, (unsigned int*)&abool);
+    }
+
+#ifdef PY3
+    res= mysql_real_connect(&self->session, host, user, password, database,
+                            port, unix_socket, client_flags);
+#else
+{
+    char* c_password;
+    if (PyUnicode_Check(password))
+    {
+        PyObject* u_password= PyUnicode_AsUTF8String(password);
+        c_password= PyString_AsString(u_password);
+        Py_DECREF(u_password);
+    }
+    else
+    {
+        c_password= PyString_AsString(password);
+    }
+    res= mysql_real_connect(&self->session, host, user, c_password, database,
+                            port, unix_socket, client_flags);
+}
+#endif
 
     Py_END_ALLOW_THREADS
 
@@ -1253,20 +1311,20 @@ MySQL_escape_string(MySQL *self, PyObject *value)
         {
             return NULL;
         }
+        from_size= BytesSize(from);
+        from_str= PyBytesAsString(from);
     }
 #ifndef PY3
     // Python v2 str
     else if (PyString_Check(value))
-    {
-        from= value;
-    }
 #else
     // Python v3 bytes
     else if (PyBytes_Check(value))
-    {
-        from= value;
-    }
 #endif
+    {
+        from_size= BytesSize(value);
+        from_str= PyBytesAsString(value);
+    }
     else
     {
 #ifdef PY3
@@ -1277,10 +1335,8 @@ MySQL_escape_string(MySQL *self, PyObject *value)
         return NULL;
     }
 
-    from_size= BytesSize(from);
     to= BytesFromStringAndSize(NULL, from_size * 2 + 1);
     to_str= PyBytesAsString(to);
-    from_str= PyBytesAsString(from);
 
     Py_BEGIN_ALLOW_THREADS
     escaped_size= (Py_ssize_t)mysql_real_escape_string(&self->session, to_str,
@@ -1289,6 +1345,7 @@ MySQL_escape_string(MySQL *self, PyObject *value)
     Py_END_ALLOW_THREADS
 
     BytesResize(&to, escaped_size);
+    Py_XDECREF(from);
 
     if (!to)
     {
