@@ -1,25 +1,30 @@
-# MySQL Connector/Python - MySQL driver written in Python.
 # Copyright (c) 2009, 2018, Oracle and/or its affiliates. All rights reserved.
-
-# MySQL Connector/Python is licensed under the terms of the GPLv2
-# <http://www.gnu.org/licenses/old-licenses/gpl-2.0.html>, like most
-# MySQL Connectors. There are special exceptions to the terms and
-# conditions of the GPLv2 as it is applied to this software, see the
-# FOSS License Exception
-# <http://www.mysql.com/about/legal/licensing/foss-exception.html>.
 #
 # This program is free software; you can redistribute it and/or modify
-# it under the terms of the GNU General Public License as published by
-# the Free Software Foundation.
+# it under the terms of the GNU General Public License, version 2.0, as
+# published by the Free Software Foundation.
 #
-# This program is distributed in the hope that it will be useful,
-# but WITHOUT ANY WARRANTY; without even the implied warranty of
-# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-# GNU General Public License for more details.
+# This program is also distributed with certain software (including
+# but not limited to OpenSSL) that is licensed under separate terms,
+# as designated in a particular file or component or in included license
+# documentation.  The authors of MySQL hereby grant you an
+# additional permission to link the program and your derivative works
+# with the separately licensed software that they have included with
+# MySQL.
+#
+# Without limiting anything contained in the foregoing, this file,
+# which is part of MySQL Connector/Python, is also subject to the
+# Universal FOSS Exception, version 1.0, a copy of which can be found at
+# http://oss.oracle.com/licenses/universal-foss-exception.
+#
+# This program is distributed in the hope that it will be useful, but
+# WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
+# See the GNU General Public License, version 2.0, for more details.
 #
 # You should have received a copy of the GNU General Public License
-# along with this program; if not, write to the Free Software
-# Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA
+# along with this program; if not, write to the Free Software Foundation, Inc.,
+# 51 Franklin St, Fifth Floor, Boston, MA 02110-1301  USA
 
 """Unittests for mysql.connector.connection
 """
@@ -35,8 +40,9 @@ import socket
 import tests
 from . import PY2
 
+from mysql.connector.connection_cext import HAVE_CMYSQL, CMySQLConnection
 from mysql.connector.conversion import (MySQLConverterBase, MySQLConverter)
-from mysql.connector import (connection, network, errors,
+from mysql.connector import (connect, connection, network, errors,
                              constants, cursor, abstracts, catch23)
 from mysql.connector.optionfiles import read_option_files
 
@@ -818,7 +824,7 @@ class MySQLConnectionTests(tests.MySQLConnectorTests):
         self.cnx._handshake['auth_plugin'] = 'caching_sha2_password'
         self.cnx._handshake['auth_data'] = b'h4i6oP!OLng9&PD@WrYH'
         self.cnx._socket.switch_to_ssl = \
-            lambda ca, cert, key, verify_cert, cipher, ssl_version: None
+            lambda ca, cert, key, verify_cert, cipher: None
 
         # Test perform_full_authentication
         # Exchange:
@@ -882,108 +888,6 @@ class MySQLConnectionTests(tests.MySQLConnectorTests):
         # Check the first auth packet
         self.assertEqual(packets[1][4:], auth_pkt)
 
-    @unittest.skipIf(not tests.SSL_AVAILABLE, "Python has no SSL support")
-    def test_auth_plugin_is_not_supported_but_required(self):
-        """Test behavior when server's default auth_plugin is required"""
-        self.cnx._socket.sock = tests.DummySocket()
-        flags = constants.ClientFlag.get_default()
-        flags |= constants.ClientFlag.SSL
-        kwargs = {
-            'username': 'ham',
-            'password': 'spam',
-            'database': 'test',
-            'charset': 33,
-            'client_flags': flags,
-            'ssl_options': {
-                'ca': os.path.join(tests.SSL_DIR, 'tests_CA_cert.pem'),
-                'cert': os.path.join(tests.SSL_DIR, 'tests_client_cert.pem'),
-                'key': os.path.join(tests.SSL_DIR, 'tests_client_key.pem'),
-            },
-        }
-
-        self.cnx._handshake['auth_plugin'] = 'unsupported_auth_plugin'
-        self.cnx._handshake['auth_data'] = b'abcdef!012345'
-        self.cnx._socket.switch_to_ssl = \
-            lambda ca, cert, key, verify_cert, cipher, ssl_version: None
-
-        # Test perform_full_authentication
-        # Exchange:
-        # Client              Server
-        # ------              ------
-        # make_ssl_auth
-        # first_auth
-        #                     Sorry user requires the unsupported_auth_plugin
-        # second_auth
-        #                     OK
-        self.cnx._socket.sock.reset()
-        self.cnx._socket.sock.add_packets([
-            bytearray(b'\x2c\x00\x00\x03\xfe\x75\x6e\x73\x75\x70\x70\x6f\x72'
-                      b'\x74\x65\x64\x5f\x61\x75\x74\x68\x5f\x70\x6c\x75\x67'
-                      b'\x69\x6e\x00\x60\x1e\x10\x78\x01\x3c\x1e\x33\x38\x6f'
-                      b'\x08\x5f\x0d\x7a\x6f\x01\x7b\x7a\x4a\x0d\x00')
-        ])
-        # Since the fictional authoritation plugin 'unsupported_auth_plugin' is
-        # not supported a NotSupportedError is raised
-        self.assertRaises(errors.NotSupportedError, self.cnx._do_auth, **kwargs)
-
-    @unittest.skipIf(not tests.SSL_AVAILABLE, "Python has no SSL support")
-    @unittest.skipIf(tests.MYSQL_VERSION < (8, 0, 3),
-                     "caching_sha2_password plugin not supported by server.")
-    def test_auth_plugin_fall_back_if_not_supported(self):
-        """Test Fall back to dflt auth_plugin if server's plugin is unknown"""
-        self.cnx._socket.sock = tests.DummySocket()
-        flags = constants.ClientFlag.get_default()
-        flags |= constants.ClientFlag.SSL
-        kwargs = {
-            'username': 'ham',
-            'password': 'spam',
-            'database': 'test',
-            'charset': 33,
-            'client_flags': flags,
-            'ssl_options': {
-                'ca': os.path.join(tests.SSL_DIR, 'tests_CA_cert.pem'),
-                'cert': os.path.join(tests.SSL_DIR, 'tests_client_cert.pem'),
-                'key': os.path.join(tests.SSL_DIR, 'tests_client_key.pem'),
-            },
-        }
-
-        self.cnx._handshake['auth_plugin'] = 'unsupported_auth_plugin'
-        self.cnx._handshake['auth_data'] = b'abcdef!012345'
-        self.cnx._socket.switch_to_ssl = \
-            lambda ca, cert, key, verify_cert, cipher, ssl_version: None
-
-        # Test perform_full_authentication
-        # Exchange:
-        # Client              Server
-        # ------              ------
-        # make_ssl_auth
-        # first_auth
-        #                     auth_switch to mysql_native_password
-        # second_auth
-        #                     OK
-        self.cnx._socket.sock.reset()
-        self.cnx._socket.sock.add_packets([
-            bytearray(b'\x8d\xaa\x0b\x00\x00\x00'), # full_auth request
-            bytearray(b'\x00@!\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00'
-                      b'\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00'),
-            bytearray(b'\x07\x00\x00\x05\x00\x00\x00\x02\x00\x00\x00') # OK
-        ])
-
-        # No exception should be raided
-        self.cnx._do_auth(**kwargs)
-
-        packets = self.cnx._socket.sock._client_sends
-
-        self.cnx._handshake['auth_plugin'] = 'mysql_native_password'
-        auth_pkt = self.cnx._protocol.make_auth(
-            self.cnx._handshake, kwargs['username'],
-            kwargs['password'], kwargs['database'],
-            charset=kwargs['charset'],
-            client_flags=kwargs['client_flags'],
-            ssl_enabled=True)
-        # Check the first_auth packet
-        self.assertEqual(packets[1][4:], auth_pkt)
-
     @unittest.skipIf(not tests.SSL_AVAILABLE, "Python has no SSL support")
     def test__do_auth_ssl(self):
         """Authenticate with the MySQL server using SSL"""
@@ -1019,7 +923,7 @@ class MySQLConnectionTests(tests.MySQLConnectorTests):
                 ssl_enabled=True),
         ]
         self.cnx._socket.switch_to_ssl = \
-            lambda ca, cert, key, verify_cert, cipher, ssl_version: None
+            lambda ca, cert, key, verify_cert, cipher: None
         self.cnx._socket.sock.reset()
         self.cnx._socket.sock.add_packets([
             bytearray(b'\x07\x00\x00\x01\x00\x00\x00\x02\x00\x00\x00'),
@@ -1803,8 +1707,6 @@ class MySQLConnectionTests(tests.MySQLConnectorTests):
             else:
                 self.assertNotEqual((b'2',), self.cnx.get_rows()[0][0])
 
-    @unittest.skipIf(tests.MYSQL_VERSION <= (5, 7, 1), "Shutdown CMD "
-                     "not tested with MySQL version 5.6 (BugOra17422299)")
     def test_shutdown(self):
         """Shutting down a connection"""
         config = tests.get_mysql_config()
@@ -1900,3 +1802,14 @@ class WL7937(tests.MySQLConnectorTests):
         sql = "LOAD DATA LOCAL INFILE %s INTO TABLE local_data"
         self.assertRaises(errors.ProgrammingError, self.cur.execute, sql,
                           (self.data_file, ))
+
+    @unittest.skipIf(HAVE_CMYSQL == False, "C Extension not available")
+    def test_use_pure(self):
+        config = tests.get_mysql_config()
+        cnx = connect(**config)
+        self.assertIsInstance(cnx, CMySQLConnection)
+        cnx.close()
+        config["use_pure"] = True
+        cnx = connect(**config)
+        self.assertIsInstance(cnx, connection.MySQLConnection)
+        cnx.close()
