@@ -1,25 +1,32 @@
 # -*- coding: utf-8 -*-
-# Copyright (c) 2009, 2016, Oracle and/or its affiliates. All rights reserved.
 
-# MySQL Connector/Python is licensed under the terms of the GPLv2
-# <http://www.gnu.org/licenses/old-licenses/gpl-2.0.html>, like most
-# MySQL Connectors. There are special exceptions to the terms and
-# conditions of the GPLv2 as it is applied to this software, see the
-# FOSS License Exception
-# <http://www.mysql.com/about/legal/licensing/foss-exception.html>.
+# Copyright (c) 2009, 2018, Oracle and/or its affiliates. All rights reserved.
 #
 # This program is free software; you can redistribute it and/or modify
-# it under the terms of the GNU General Public License as published by
-# the Free Software Foundation.
+# it under the terms of the GNU General Public License, version 2.0, as
+# published by the Free Software Foundation.
 #
-# This program is distributed in the hope that it will be useful,
-# but WITHOUT ANY WARRANTY; without even the implied warranty of
-# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-# GNU General Public License for more details.
+# This program is also distributed with certain software (including
+# but not limited to OpenSSL) that is licensed under separate terms,
+# as designated in a particular file or component or in included license
+# documentation.  The authors of MySQL hereby grant you an
+# additional permission to link the program and your derivative works
+# with the separately licensed software that they have included with
+# MySQL.
+#
+# Without limiting anything contained in the foregoing, this file,
+# which is part of MySQL Connector/Python, is also subject to the
+# Universal FOSS Exception, version 1.0, a copy of which can be found at
+# http://oss.oracle.com/licenses/universal-foss-exception.
+#
+# This program is distributed in the hope that it will be useful, but
+# WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
+# See the GNU General Public License, version 2.0, for more details.
 #
 # You should have received a copy of the GNU General Public License
-# along with this program; if not, write to the Free Software
-# Foundation, Incur., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA
+# along with this program; if not, write to the Free Software Foundation, Inc.,
+# 51 Franklin St, Fifth Floor, Boston, MA 02110-1301  USA
 
 """Test module for bugs
 
@@ -321,7 +328,7 @@ class MySQLCursorTests(tests.TestsCursor):
         self.cur = cursor.MySQLCursor(self.cnx)
         self.assertRaises(StopIteration, self.cur.__next__)
         self.cur.execute("SELECT BINARY 'ham'")
-        exp = (b'ham',)
+        exp = ('ham',)
         self.assertEqual(exp, next(self.cur))
         self.cur.close()
 
@@ -554,7 +561,7 @@ class MySQLCursorTests(tests.TestsCursor):
         self.assertTrue(tests.cmp_result(exp, self.cur._warnings))
 
         self.cur.execute("SELECT BINARY 'ham'")
-        exp = [(b'ham',)]
+        exp = [('ham',)]
         self.assertEqual(exp, self.cur.fetchall())
         self.cur.close()
 
@@ -826,7 +833,7 @@ class MySQLCursorTests(tests.TestsCursor):
         self.cnx = connection.MySQLConnection(**tests.get_mysql_config())
         self.cur = self.cnx.cursor()
         self.cur.execute("SELECT BINARY 'ham'")
-        exp = (b'ham',)
+        exp = ('ham',)
         self.assertEqual(exp, self.cur.fetchone())
         self.assertEqual(None, self.cur.fetchone())
         self.cur.close()
@@ -1113,7 +1120,7 @@ class MySQLCursorPreparedTests(tests.TestsCursor):
 
     def test_callproc(self):
         cur = self.cnx.cursor(cursor_class=cursor.MySQLCursorPrepared)
-        self.assertRaises(errors.NotSupportedError, cur.callproc)
+        self.assertRaises(errors.NotSupportedError, cur.callproc, None)
 
     def test_close(self):
         cur = self.cnx.cursor(cursor_class=cursor.MySQLCursorPrepared)
@@ -1232,7 +1239,7 @@ class MySQLCursorPreparedTests(tests.TestsCursor):
         self.assertEqual(3, cur.rowcount)
 
         cur.execute(stmt_select)
-        self.assertEqual([(1, b'100'), (2, b'200'), (3, b'300')],
+        self.assertEqual([(1, '100'), (2, '200'), (3, '300')],
                          cur.fetchall(), "Multi insert test failed")
 
         data = [(2,), (3,)]
@@ -1255,8 +1262,8 @@ class MySQLCursorPreparedTests(tests.TestsCursor):
             return row
         cur._fetch_row = _fetch_row
 
-        cur._test_fetch_row = [(b'ham',)]
-        self.assertEqual((b'ham',), cur.fetchone())
+        cur._test_fetch_row = [('ham',)]
+        self.assertEqual(('ham',), cur.fetchone())
         self.assertEqual(None, cur.fetchone())
 
     def test_fetchmany(self):
