@@ -1,25 +1,30 @@
-# MySQL Connector/Python - MySQL driver written in Python.
-# Copyright (c) 2014, 2015, Oracle and/or its affiliates. All rights reserved.
-
-# MySQL Connector/Python is licensed under the terms of the GPLv2
-# <http://www.gnu.org/licenses/old-licenses/gpl-2.0.html>, like most
-# MySQL Connectors. There are special exceptions to the terms and
-# conditions of the GPLv2 as it is applied to this software, see the
-# FOSS License Exception
-# <http://www.mysql.com/about/legal/licensing/foss-exception.html>.
+# Copyright (c) 2014, 2017, Oracle and/or its affiliates. All rights reserved.
 #
 # This program is free software; you can redistribute it and/or modify
-# it under the terms of the GNU General Public License as published by
-# the Free Software Foundation.
+# it under the terms of the GNU General Public License, version 2.0, as
+# published by the Free Software Foundation.
 #
-# This program is distributed in the hope that it will be useful,
-# but WITHOUT ANY WARRANTY; without even the implied warranty of
-# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-# GNU General Public License for more details.
+# This program is also distributed with certain software (including
+# but not limited to OpenSSL) that is licensed under separate terms,
+# as designated in a particular file or component or in included license
+# documentation.  The authors of MySQL hereby grant you an
+# additional permission to link the program and your derivative works
+# with the separately licensed software that they have included with
+# MySQL.
+#
+# Without limiting anything contained in the foregoing, this file,
+# which is part of MySQL Connector/Python, is also subject to the
+# Universal FOSS Exception, version 1.0, a copy of which can be found at
+# http://oss.oracle.com/licenses/universal-foss-exception.
+#
+# This program is distributed in the hope that it will be useful, but
+# WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
+# See the GNU General Public License, version 2.0, for more details.
 #
 # You should have received a copy of the GNU General Public License
-# along with this program; if not, write to the Free Software
-# Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA
+# along with this program; if not, write to the Free Software Foundation, Inc.,
+# 51 Franklin St, Fifth Floor, Boston, MA 02110-1301  USA
 
 """Unittests for mysql.connector.django
 """
@@ -60,6 +65,7 @@ if DJANGO_AVAILABLE:
             'TEST_COLLATION': 'utf8_general_ci',
             'CONN_MAX_AGE': 0,
             'AUTOCOMMIT': True,
+            'TIME_ZONE': None,
         },
     }
     settings.SECRET_KEY = "django_tests_secret_key"
@@ -92,15 +98,11 @@ FOREIGN KEY (id_t1) REFERENCES django_t1(id) ON DELETE CASCADE
 
 # Have to load django.db to make importing db backend work for Django < 1.6
 import django.db  # pylint: disable=W0611
-if tests.DJANGO_VERSION >= (1, 6):
-    if tests.DJANGO_VERSION >= (1, 8):
-        from django.db.backends.base.introspection import FieldInfo
-    else:
-        from django.db.backends import FieldInfo
 from django.db.backends.signals import connection_created
 from django.utils.safestring import SafeBytes, SafeText
 
 import mysql.connector
+from mysql.connector.django.introspection import FieldInfo
 
 if DJANGO_AVAILABLE:
     from mysql.connector.django.base import (
@@ -147,14 +149,18 @@ class DjangoIntrospection(tests.MySQLConnectorTests):
 
     def test_get_table_list(self):
         cur = self.cnx.cursor()
-        exp = list(TABLES.keys())
-        for exp in list(TABLES.keys()):
+        for exp in TABLES.keys():
             if sys.version_info < (2, 7):
                 self.assertTrue(exp in self.introspect.get_table_list(cur))
             else:
-                self.assertIn(exp, self.introspect.get_table_list(cur),
-                              "Table {table_name} not in table list".format(
-                                  table_name=exp))
+                if tests.DJANGO_VERSION < (1, 8):
+                    res = any(table == exp
+                              for table in self.introspect.get_table_list(cur))
+                else:
+                    res = any(table.name == exp
+                              for table in self.introspect.get_table_list(cur))
+                self.assertTrue(res, "Table {table_name} not in table list"
+                                     "".format(table_name=exp))
 
     def test_get_table_description(self):
         cur = self.cnx.cursor()
@@ -165,24 +171,51 @@ class DjangoIntrospection(tests.MySQLConnectorTests):
                 ('c1', 3, None, None, None, None, 1, 16392),
                 ('c2', 253, None, 20, None, None, 1, 16388)
             ]
-        else:
+        elif tests.DJANGO_VERSION < (1, 8):
             exp = [
-                FieldInfo(name='id', type_code=3, display_size=None,
+                FieldInfo(name=u'id', type_code=3, display_size=None,
                           internal_size=None, precision=None, scale=None,
                           null_ok=0),
-                FieldInfo(name='c1', type_code=3, display_size=None,
+                FieldInfo(name=u'c1', type_code=3, display_size=None,
                           internal_size=None, precision=None, scale=None,
                           null_ok=1),
-                FieldInfo(name='c2', type_code=253, display_size=None,
+                FieldInfo(name=u'c2', type_code=253, display_size=None,
                           internal_size=20, precision=None, scale=None,
                           null_ok=1)
             ]
+        elif tests.DJANGO_VERSION < (1, 11):
+            exp = [
+                FieldInfo(name=u'id', type_code=3, display_size=None,
+                          internal_size=None, precision=10, scale=None,
+                          null_ok=0, extra=u'auto_increment'),
+                FieldInfo(name=u'c1', type_code=3, display_size=None,
+                          internal_size=None, precision=10, scale=None,
+                          null_ok=1, extra=u''),
+                FieldInfo(name=u'c2', type_code=253, display_size=None,
+                          internal_size=20, precision=None, scale=None,
+                          null_ok=1, extra=u'')
+            ]
+        else:
+            exp = [
+                FieldInfo(name=u'id', type_code=3, display_size=None,
+                          internal_size=None, precision=10, scale=None,
+                          null_ok=0, default=None, extra=u'auto_increment'),
+                FieldInfo(name=u'c1', type_code=3, display_size=None,
+                          internal_size=None, precision=10, scale=None,
+                          null_ok=1, default=None, extra=u''),
+                FieldInfo(name=u'c2', type_code=253, display_size=None,
+                          internal_size=20, precision=None, scale=None,
+                          null_ok=1, default=None, extra=u'')
+            ]
         res = self.introspect.get_table_description(cur, 'django_t1')
         self.assertEqual(exp, res)
 
     def test_get_relations(self):
         cur = self.cnx.cursor()
-        exp = {1: (0, 'django_t1')}
+        if tests.DJANGO_VERSION < (1, 8):
+            exp = {1: (0, 'django_t1')}
+        else:
+            exp = {u'id_t1': (u'id', u'django_t1')}
         self.assertEqual(exp, self.introspect.get_relations(cur, 'django_t2'))
 
     def test_get_key_columns(self):
@@ -204,6 +237,30 @@ class DjangoIntrospection(tests.MySQLConnectorTests):
         res = self.introspect.get_primary_key_column(cur, 'django_t1')
         self.assertEqual('id', res)
 
+    def test_get_constraints(self):
+        cur = self.cnx.cursor()
+        exp = {
+            'PRIMARY': {'check': False,
+                        'columns': ['id'],
+                        'foreign_key': None,
+                        'index': True,
+                        'primary_key': True,
+                        'unique': True},
+            'django_t2_ibfk_1': {'check': False,
+                                 'columns': ['id_t1'],
+                                 'foreign_key': ('django_t1', 'id'),
+                                 'index': False,
+                                 'primary_key': False,
+                                 'unique': False},
+            'id_t1': {'check': False,
+                      'columns': ['id_t1'],
+                      'foreign_key': None,
+                      'index': True,
+                      'primary_key': False,
+                      'unique': False}
+        }
+        self.assertEqual(
+            exp, self.introspect.get_constraints(cur, 'django_t2'))
 
 @unittest.skipIf(not DJANGO_AVAILABLE, "Django not available")
 class DjangoDatabaseWrapper(tests.MySQLConnectorTests):
@@ -277,26 +334,49 @@ class DjangoDatabaseOperations(tests.MySQLConnectorTests):
         self.dbo = DatabaseOperations(self.cnx)
 
     def test_value_to_db_time(self):
-        self.assertEqual(None, self.dbo.value_to_db_time(None))
+        if tests.DJANGO_VERSION < (1, 9):
+            value_to_db_time = self.dbo.value_to_db_time
+        else:
+            value_to_db_time = self.dbo.adapt_timefield_value
+
+        self.assertEqual(None, value_to_db_time(None))
 
         value = datetime.time(0, 0, 0)
         exp = self.conn.converter._time_to_mysql(value)
-        self.assertEqual(exp, self.dbo.value_to_db_time(value))
+        self.assertEqual(exp, value_to_db_time(value))
 
         value = datetime.time(2, 5, 7)
         exp = self.conn.converter._time_to_mysql(value)
-        self.assertEqual(exp, self.dbo.value_to_db_time(value))
+        self.assertEqual(exp, value_to_db_time(value))
 
     def test_value_to_db_datetime(self):
-        self.assertEqual(None, self.dbo.value_to_db_datetime(None))
+        if tests.DJANGO_VERSION < (1, 9):
+            value_to_db_datetime = self.dbo.value_to_db_datetime
+        else:
+            value_to_db_datetime = self.dbo.adapt_datetimefield_value
+
+        self.assertEqual(None, value_to_db_datetime(None))
 
         value = datetime.datetime(1, 1, 1)
         exp = self.conn.converter._datetime_to_mysql(value)
-        self.assertEqual(exp, self.dbo.value_to_db_datetime(value))
+        self.assertEqual(exp, value_to_db_datetime(value))
 
         value = datetime.datetime(2, 5, 7, 10, 10)
         exp = self.conn.converter._datetime_to_mysql(value)
-        self.assertEqual(exp, self.dbo.value_to_db_datetime(value))
+        self.assertEqual(exp, value_to_db_datetime(value))
+
+    def test_bulk_insert_sql(self):
+        num_values = 5
+        fields = ["col1", "col2", "col3"]
+        placeholder_rows = [["%s"] * len(fields) for _ in range(num_values)]
+        exp = "VALUES {0}".format(", ".join(
+            ["({0})".format(", ".join(["%s"] * len(fields)))] * num_values))
+        if tests.DJANGO_VERSION < (1, 9):
+            self.assertEqual(
+                exp, self.dbo.bulk_insert_sql(fields, num_values))
+        else:
+            self.assertEqual(
+                exp, self.dbo.bulk_insert_sql(fields, placeholder_rows))
 
 
 class DjangoMySQLConverterTests(tests.MySQLConnectorTests):
