@@ -1,27 +1,33 @@
 #!/usr/bin/env python
 # -*- coding: utf-8 -*-
-# MySQL Connector/Python - MySQL driver written in Python.
-# Copyright (c) 2009, 2015, Oracle and/or its affiliates. All rights reserved.
 
-# MySQL Connector/Python is licensed under the terms of the GPLv2
-# <http://www.gnu.org/licenses/old-licenses/gpl-2.0.html>, like most
-# MySQL Connectors. There are special exceptions to the terms and
-# conditions of the GPLv2 as it is applied to this software, see the
-# FOSS License Exception
-# <http://www.mysql.com/about/legal/licensing/foss-exception.html>.
+# Copyright (c) 2009, 2017, Oracle and/or its affiliates. All rights reserved.
 #
 # This program is free software; you can redistribute it and/or modify
-# it under the terms of the GNU General Public License as published by
-# the Free Software Foundation.
+# it under the terms of the GNU General Public License, version 2.0, as
+# published by the Free Software Foundation.
 #
-# This program is distributed in the hope that it will be useful,
-# but WITHOUT ANY WARRANTY; without even the implied warranty of
-# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-# GNU General Public License for more details.
+# This program is also distributed with certain software (including
+# but not limited to OpenSSL) that is licensed under separate terms,
+# as designated in a particular file or component or in included license
+# documentation.  The authors of MySQL hereby grant you an
+# additional permission to link the program and your derivative works
+# with the separately licensed software that they have included with
+# MySQL.
+#
+# Without limiting anything contained in the foregoing, this file,
+# which is part of MySQL Connector/Python, is also subject to the
+# Universal FOSS Exception, version 1.0, a copy of which can be found at
+# http://oss.oracle.com/licenses/universal-foss-exception.
+#
+# This program is distributed in the hope that it will be useful, but
+# WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
+# See the GNU General Public License, version 2.0, for more details.
 #
 # You should have received a copy of the GNU General Public License
-# along with this program; if not, write to the Free Software
-# Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA
+# along with this program; if not, write to the Free Software Foundation, Inc.,
+# 51 Franklin St, Fifth Floor, Boston, MA 02110-1301  USA
 
 """Script for running unittests
 
@@ -99,9 +105,15 @@ else:
 # MySQL option file template. Platform specifics dynamically added later.
 MY_CNF = """
 # MySQL option file for MySQL Connector/Python tests
+[mysqld-8.0]
+plugin-load={mysqlx_plugin}
+loose_mysqlx_port={mysqlx_port}
+{mysqlx_bind_address}
+
 [mysqld-5.7]
 plugin-load={mysqlx_plugin}
 loose_mysqlx_port={mysqlx_port}
+{mysqlx_bind_address}
 
 [mysqld-5.6]
 innodb_compression_level = 0
@@ -141,27 +153,28 @@ local_infile = 1
 innodb_flush_log_at_trx_commit = 2
 innodb_log_file_size = 1Gb
 general_log_file = general_{name}.log
-ssl
 """
 
 # Platform specifics
 if os.name == 'nt':
     MY_CNF += '\n'.join((
-        "ssl-ca = {ssl_dir}\\\\tests_CA_cert.pem",
-        "ssl-cert = {ssl_dir}\\\\tests_server_cert.pem",
-        "ssl-key = {ssl_dir}\\\\tests_server_key.pem",
+        "ssl-ca = {ssl_ca}",
+        "ssl-cert = {ssl_cert}",
+        "ssl-key = {ssl_key}",
     ))
     MYSQL_DEFAULT_BASE = os.path.join(
         "C:/", "Program Files", "MySQL", "MySQL Server 5.6")
 else:
     MY_CNF += '\n'.join((
-        "ssl-ca = {ssl_dir}/tests_CA_cert.pem",
-        "ssl-cert = {ssl_dir}/tests_server_cert.pem",
-        "ssl-key = {ssl_dir}/tests_server_key.pem",
+        "ssl-ca = {ssl_ca}",
+        "ssl-cert = {ssl_cert}",
+        "ssl-key = {ssl_key}",
         "innodb_flush_method = O_DIRECT",
     ))
     MYSQL_DEFAULT_BASE = os.path.join('/', 'usr', 'local', 'mysql')
 
+MY_CNF += "\nssl={ssl}"
+
 MYSQL_DEFAULT_TOPDIR = _TOPDIR
 
 _UNITTESTS_CMD_ARGS = {
@@ -328,6 +341,24 @@ _UNITTESTS_CMD_ARGS = {
                  "or full path to mysql_config")
     },
 
+    ('', '--with-protobuf-include-dir'): {
+        'dest': 'protobuf_include_dir', 'metavar': 'NAME',
+        'default': None,
+        'help': ("Location of Protobuf include directory")
+    },
+
+    ('', '--with-protobuf-lib-dir'): {
+        'dest': 'protobuf_lib_dir', 'metavar': 'NAME',
+        'default': None,
+        'help': ("Location of Protobuf library directory")
+    },
+
+    ('', '--with-protoc'): {
+        'dest': 'protoc', 'metavar': 'NAME',
+        'default': None,
+        'help': ("Location of Protobuf protoc binary")
+    },
+
     ('', '--with-fabric'): {
         'dest': 'fabric_config', 'metavar': 'NAME',
         'default': None,
@@ -339,6 +370,18 @@ _UNITTESTS_CMD_ARGS = {
         'default': 'xmlrpc',
         'help': ("Protocol to talk to MySQL Fabric")
     },
+
+    ('', '--extra-compile-args'): {
+        'dest': 'extra_compile_args', 'metavar': 'NAME',
+        'default': None,
+        'help': ("Extra compile args for the C extension")
+    },
+
+    ('', '--extra-link-args'): {
+        'dest': 'extra_link_args', 'metavar': 'NAME',
+        'default': None,
+        'help': ("Extra link args for the C extension")
+    },
 }
 
 
@@ -602,6 +645,11 @@ def setup_stats_db(cnx):
 def init_mysql_server(port, options):
     """Initialize a MySQL Server"""
     name = 'server{0}'.format(len(tests.MYSQL_SERVERS) + 1)
+    extra_args = [{
+        "version": (5, 7, 17),
+        "options": {"mysqlx_bind_address": "mysqlx_bind_address={0}".format("::"
+                    if tests.IPV6_AVAILABLE else "0.0.0.0")}
+    }]
 
     try:
         mysql_server = mysqld.MySQLServer(
@@ -613,7 +661,11 @@ def init_mysql_server(port, options):
             mysqlx_port=options.mysqlx_port,
             unix_socket_folder=options.unix_socket_folder,
             ssl_folder=os.path.abspath(tests.SSL_DIR),
+            ssl_ca="tests_CA_cert.pem",
+            ssl_cert="tests_server_cert.pem",
+            ssl_key="tests_server_key.pem",
             name=name,
+            extra_args=extra_args,
             sharedir=options.mysql_sharedir)
     except tests.mysqld.MySQLBootstrapError as err:
         LOGGER.error("Failed initializing MySQL server "
@@ -776,8 +828,28 @@ def main():
 
     tests.MYSQL_CAPI = options.mysql_capi
     if not options.skip_install:
+        protobuf_include_dir = options.protobuf_include_dir or \
+            os.environ.get("MYSQLXPB_PROTOBUF_INCLUDE_DIR")
+        protobuf_lib_dir = options.protobuf_lib_dir or \
+            os.environ.get("MYSQLXPB_PROTOBUF_LIB_DIR")
+        protoc = options.protoc or os.environ.get("MYSQLXPB_PROTOC")
+        if any((protobuf_include_dir, protobuf_lib_dir, protoc)):
+            if not protobuf_include_dir:
+                LOGGER.error("Unable to find Protobuf include directory.")
+                sys.exit(1)
+            if not protobuf_lib_dir:
+                LOGGER.error("Unable to find Protobuf library directory.")
+                sys.exit(1)
+            if not protoc:
+                LOGGER.error("Unable to find Protobuf protoc binary.")
+                sys.exit(1)
         tests.install_connector(_TOPDIR, tests.TEST_BUILD_DIR,
-                                options.mysql_capi)
+                                protobuf_include_dir,
+                                protobuf_lib_dir,
+                                protoc,
+                                options.mysql_capi,
+                                options.extra_compile_args,
+                                options.extra_link_args)
 
     # Which tests cases to run
     testcases = []
